window.Gleam.initSearch([{"doc":"bigi","title":"bigi","content":"# bigi\n\n[![Package Version](https://img.shields.io/hexpm/v/bigi)](https://hex.pm/packages/bigi)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/bigi/)\n\nArbitrary precision integer arithmetic for Gleam.\n\nIn the Erlang target, all integers are automatically &quot;big integers&quot; that are subject to\narbitrary precision arithmetic. This means there is no need for this package if you are only\ntargeting Erlang.\n\nIn JavaScript, Gleam&#39;s `Int` type corresponds to the `number` type, which is implemented using\nfloating-point arithmetic. That means it&#39;s subject to the following limits:\n\n- [Maximum safe integer](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MAX_SAFE_INTEGER)\n  and\n  [minimum safe integer](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MIN_SAFE_INTEGER)\n  beyond which the accuracy of arithmetic suffers, and\n- [maximum value](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MAX_VALUE)\n  and\n  [minimum value](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MIN_VALUE)\n  beyond which the numbers are converted to `Infinity` or `-Infinity`.\n\nThis package thus provides big integers for the JavaScript target and additionally provides a\nconsistent interface for packages that target both Erlang and JavaScript. In Erlang, regular\nintegers are used. In JavaScript, the\n[`BigInt`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/BigInt)\ntype is used.\n\n```sh\ngleam add bigi\n```\n\n```gleam\nimport bigi\n\npub fn main() {\n  bigi.power(\n    bigi.from_int(2),\n    bigi.from_int(65_535)\n  )\n\n  // Ok(1001764965203423232489536175780127875223912737784875709632508486855447029778...)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/bigi&gt;.\n\n## Limitations\n\nErlang does have [a limit for big integers at around 4 megabits](https://elixirforum.com/t/is-there-an-integer-size-limit/65647/8?u=nicd),\nas demonstrated below:\n\n```gleam\nimport bigi\n\npub fn main() {\n  // Works\n  bigi.power(bigi.from_int(2), bigi.from_int(4_194_239))\n\n  // Blows up\n  bigi.power(bigi.from_int(2), bigi.from_int(4_194_240))\n}\n```\n\nFor Node.js, the limit is somewhere higher.\n\n`bigi` has no facilities to warn about or handle errors due to these system limits. It is up to the developer\nto be mindful of them.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"bigi","title":"BigInt","content":"pub type BigInt\n A big integer.\n\nSynonyms:\nbigi.BigInt\nbigi BigInt","url":"bigi.html#BigInt"},{"doc":"bigi","title":"Endianness","content":"pub type Endianness {\n  LittleEndian\n  BigEndian\n}\n Endianness specifier used when converting between a big integer and\n raw bytes.\nLittleEndian\n\n\nBigEndian\n\n\nSynonyms:\nbigi.Endianness\nbigi Endianness","url":"bigi.html#Endianness"},{"doc":"bigi","title":"Signedness","content":"pub type Signedness {\n  Signed\n  Unsigned\n}\n Signedness specifier used when converting between a big integer and\n raw bytes.\nSigned\n\n\nUnsigned\n\n\nSynonyms:\nbigi.Signedness\nbigi Signedness","url":"bigi.html#Signedness"},{"doc":"bigi","title":"absolute","content":"pub fn absolute(bigint: BigInt) -&gt; BigInt\n Get the absolute value of a big integer.\nSynonyms:\nbigi.absolute\nbigi absolute","url":"bigi.html#absolute"},{"doc":"bigi","title":"add","content":"pub fn add(a: BigInt, b: BigInt) -&gt; BigInt\n Add two big integers together.\nSynonyms:\nbigi.add\nbigi add","url":"bigi.html#add"},{"doc":"bigi","title":"bitwise_and","content":"pub fn bitwise_and(a: BigInt, b: BigInt) -&gt; BigInt\n Calculates the bitwise AND of its arguments.\nSynonyms:\nbigi.bitwise_and\nbigi bitwise_and","url":"bigi.html#bitwise_and"},{"doc":"bigi","title":"bitwise_exclusive_or","content":"pub fn bitwise_exclusive_or(a: BigInt, b: BigInt) -&gt; BigInt\n Calculates the bitwise XOR of its arguments.\nSynonyms:\nbigi.bitwise_exclusive_or\nbigi bitwise_exclusive_or","url":"bigi.html#bitwise_exclusive_or"},{"doc":"bigi","title":"bitwise_not","content":"pub fn bitwise_not(bigint: BigInt) -&gt; BigInt\n Calculates the bitwise NOT of its argument.\nSynonyms:\nbigi.bitwise_not\nbigi bitwise_not","url":"bigi.html#bitwise_not"},{"doc":"bigi","title":"bitwise_or","content":"pub fn bitwise_or(a: BigInt, b: BigInt) -&gt; BigInt\n Calculates the bitwise OR of its arguments.\nSynonyms:\nbigi.bitwise_or\nbigi bitwise_or","url":"bigi.html#bitwise_or"},{"doc":"bigi","title":"bitwise_shift_left","content":"pub fn bitwise_shift_left(\n  bigint: BigInt,\n  by amount: Int,\n) -&gt; BigInt\n Calculates the result of an arithmetic left bitshift by the given amount.\nSynonyms:\nbigi.bitwise_shift_left\nbigi bitwise_shift_left","url":"bigi.html#bitwise_shift_left"},{"doc":"bigi","title":"bitwise_shift_right","content":"pub fn bitwise_shift_right(\n  bigint: BigInt,\n  by amount: Int,\n) -&gt; BigInt\n Calculates the result of an arithmetic right bitshift by the given amount.\nSynonyms:\nbigi.bitwise_shift_right\nbigi bitwise_shift_right","url":"bigi.html#bitwise_shift_right"},{"doc":"bigi","title":"clamp","content":"pub fn clamp(\n  bigint: BigInt,\n  min min_bound: BigInt,\n  max max_bound: BigInt,\n) -&gt; BigInt\n Restricts a big integer between a lower and upper bound.\nSynonyms:\nbigi.clamp\nbigi clamp","url":"bigi.html#clamp"},{"doc":"bigi","title":"compare","content":"pub fn compare(a: BigInt, with b: BigInt) -&gt; Order\n Compare two big integers, returning an order that denotes if `a` is lower,\n bigger than, or equal to `b`.\nSynonyms:\nbigi.compare\nbigi compare","url":"bigi.html#compare"},{"doc":"bigi","title":"decode","content":"pub fn decode(dyn: Dynamic) -&gt; Result(BigInt, List(DecodeError))\n Decode a `gleam/dynamic` value into a big integer, if possible.\nSynonyms:\nbigi.decode\nbigi decode","url":"bigi.html#decode"},{"doc":"bigi","title":"digits","content":"pub fn digits(bigint: BigInt) -&gt; List(Int)\n Get the digits in a given bigint as a list of integers in base 10.\n\n The list is ordered starting from the most significant digit.\nSynonyms:\nbigi.digits\nbigi digits","url":"bigi.html#digits"},{"doc":"bigi","title":"divide","content":"pub fn divide(dividend a: BigInt, divisor b: BigInt) -&gt; BigInt\n Divide the dividend with the divisor using integer division.\n\n Follows the standard Gleam divide-by-zero rule of 0 when the divisor is 0.\nSynonyms:\nbigi.divide\nbigi divide","url":"bigi.html#divide"},{"doc":"bigi","title":"divide_no_zero","content":"pub fn divide_no_zero(\n  dividend a: BigInt,\n  divisor b: BigInt,\n) -&gt; Result(BigInt, Nil)\n Divide the dividend with the divisor using integer division.\n\n Returns an error if the divisor is 0.\nSynonyms:\nbigi.divide_no_zero\nbigi divide_no_zero","url":"bigi.html#divide_no_zero"},{"doc":"bigi","title":"floor_divide","content":"pub fn floor_divide(\n  dividend dividend: BigInt,\n  divisor divisor: BigInt,\n) -&gt; Result(BigInt, Nil)\n Performs a *floored* integer division, which means that the result will\n always be rounded towards negative infinity.\n\n If you want to perform truncated integer division (rounding towards zero),\n use `divide` or `divide_no_zero` instead.\n\n Returns an error if the divisor is 0.\nSynonyms:\nbigi.floor_divide\nbigi floor_divide","url":"bigi.html#floor_divide"},{"doc":"bigi","title":"from_bytes","content":"pub fn from_bytes(\n  bytes: BitArray,\n  endianness: Endianness,\n  signedness: Signedness,\n) -&gt; Result(BigInt, Nil)\n Convert raw bytes into a big integer.\n \n If the bit array does not contain a whole number of bytes then an error is\n returned.\nSynonyms:\nbigi.from_bytes\nbigi from_bytes","url":"bigi.html#from_bytes"},{"doc":"bigi","title":"from_int","content":"pub fn from_int(int: Int) -&gt; BigInt\n Create a big integer from a regular integer.\n\n Note that in the JavaScript target, if your integer is bigger than the\n [maximum safe integer](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MAX_SAFE_INTEGER)\n or smaller than the\n [minimum safe integer](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MIN_SAFE_INTEGER),\n you may lose precision when operating on it, including when converting it\n into a big integer (as the JavaScript Number type has already reduced the\n precision of the value).\nSynonyms:\nbigi.from_int\nbigi from_int","url":"bigi.html#from_int"},{"doc":"bigi","title":"from_string","content":"pub fn from_string(str: String) -&gt; Result(BigInt, Nil)\n Convert a string into a big integer.\n\n If the string does not represent a big integer in base 10, an error is\n returned. Trailing non-digit content is not allowed.\nSynonyms:\nbigi.from_string\nbigi from_string","url":"bigi.html#from_string"},{"doc":"bigi","title":"is_odd","content":"pub fn is_odd(bigint: BigInt) -&gt; Bool\n Returns whether the big integer provided is odd.\nSynonyms:\nbigi.is_odd\nbigi is_odd","url":"bigi.html#is_odd"},{"doc":"bigi","title":"max","content":"pub fn max(a: BigInt, or b: BigInt) -&gt; BigInt\n Compares two big integers, returning the larger of the two.\nSynonyms:\nbigi.max\nbigi max","url":"bigi.html#max"},{"doc":"bigi","title":"min","content":"pub fn min(a: BigInt, or b: BigInt) -&gt; BigInt\n Compares two big integers, returning the smaller of the two.\nSynonyms:\nbigi.min\nbigi min","url":"bigi.html#min"},{"doc":"bigi","title":"modulo","content":"pub fn modulo(dividend a: BigInt, divisor b: BigInt) -&gt; BigInt\n Calculate a mathematical modulo operation.\n\n Follows the standard Gleam divide-by-zero rule of 0 when the divisor is 0.\nSynonyms:\nbigi.modulo\nbigi modulo","url":"bigi.html#modulo"},{"doc":"bigi","title":"modulo_no_zero","content":"pub fn modulo_no_zero(\n  dividend a: BigInt,\n  divisor b: BigInt,\n) -&gt; Result(BigInt, Nil)\n Calculate a mathematical modulo operation.\n\n Returns an error if the divisor is 0.\nSynonyms:\nbigi.modulo_no_zero\nbigi modulo_no_zero","url":"bigi.html#modulo_no_zero"},{"doc":"bigi","title":"multiply","content":"pub fn multiply(a: BigInt, b: BigInt) -&gt; BigInt\n Multiply two big integers together.\nSynonyms:\nbigi.multiply\nbigi multiply","url":"bigi.html#multiply"},{"doc":"bigi","title":"negate","content":"pub fn negate(bigint: BigInt) -&gt; BigInt\n Returns the negative of the value provided.\nSynonyms:\nbigi.negate\nbigi negate","url":"bigi.html#negate"},{"doc":"bigi","title":"power","content":"pub fn power(\n  base a: BigInt,\n  exponent b: BigInt,\n) -&gt; Result(BigInt, Nil)\n Raise the base to the exponent.\n\n If the exponent is negative, an error is returned.\nSynonyms:\nbigi.power\nbigi power","url":"bigi.html#power"},{"doc":"bigi","title":"product","content":"pub fn product(bigints: List(BigInt)) -&gt; BigInt\n Multiplies a list of big integers.\n\n Returns 1 if the list was empty.\nSynonyms:\nbigi.product\nbigi product","url":"bigi.html#product"},{"doc":"bigi","title":"remainder","content":"pub fn remainder(dividend a: BigInt, divisor b: BigInt) -&gt; BigInt\n Divide the dividend with the divisor using integer division and return the\n remainder.\n\n Follows the standard Gleam divide-by-zero rule of 0 when the divisor is 0.\nSynonyms:\nbigi.remainder\nbigi remainder","url":"bigi.html#remainder"},{"doc":"bigi","title":"remainder_no_zero","content":"pub fn remainder_no_zero(\n  dividend a: BigInt,\n  divisor b: BigInt,\n) -&gt; Result(BigInt, Nil)\n Divide the dividend with the divisor using integer division and return the\n remainder.\n\n Returns an error if the divisor is 0.\nSynonyms:\nbigi.remainder_no_zero\nbigi remainder_no_zero","url":"bigi.html#remainder_no_zero"},{"doc":"bigi","title":"subtract","content":"pub fn subtract(\n  minuend a: BigInt,\n  subtrahend b: BigInt,\n) -&gt; BigInt\n Subtract the subtrahend from the minuend.\nSynonyms:\nbigi.subtract\nbigi subtract","url":"bigi.html#subtract"},{"doc":"bigi","title":"sum","content":"pub fn sum(bigints: List(BigInt)) -&gt; BigInt\n Sums a list of big integers.\n\n Returns 0 if the list was empty.\nSynonyms:\nbigi.sum\nbigi sum","url":"bigi.html#sum"},{"doc":"bigi","title":"to_bytes","content":"pub fn to_bytes(\n  bigint: BigInt,\n  endianness: Endianness,\n  signedness: Signedness,\n  byte_count: Int,\n) -&gt; Result(BitArray, Nil)\n Convert a big integer to raw bytes.\n \n The size of the returned bit array is specified by `byte_count`, e.g. 8 will\n return a bit array containing 8 bytes (64 bits). If the big integer doesn&#39;t\n fit in the specified number of bytes then an error is returned.\nSynonyms:\nbigi.to_bytes\nbigi to_bytes","url":"bigi.html#to_bytes"},{"doc":"bigi","title":"to_int","content":"pub fn to_int(bigint: BigInt) -&gt; Result(Int, Nil)\n Convert a big integer to a regular integer.\n\n In Erlang, this cannot fail, as all Erlang integers are big integers. In the\n JavaScript target, this will fail if the integer is bigger than the\n [maximum safe integer](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MAX_SAFE_INTEGER)\n or smaller than the\n [minimum safe integer](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MIN_SAFE_INTEGER).\nSynonyms:\nbigi.to_int\nbigi to_int","url":"bigi.html#to_int"},{"doc":"bigi","title":"to_string","content":"pub fn to_string(bigint: BigInt) -&gt; String\n Convert the big integer into a simple string - a sequence of digits.\nSynonyms:\nbigi.to_string\nbigi to_string","url":"bigi.html#to_string"},{"doc":"bigi","title":"undigits","content":"pub fn undigits(\n  digits: List(Int),\n  base: Int,\n) -&gt; Result(BigInt, Nil)\n Joins a list of digits into a single value. Returns an error if the base is\n less than 2 or if the list contains a digit greater than or equal to the\n specified base.\nSynonyms:\nbigi.undigits\nbigi undigits","url":"bigi.html#undigits"},{"doc":"bigi","title":"zero","content":"pub fn zero() -&gt; BigInt\n Create a big integer representing zero.\nSynonyms:\nbigi.zero\nbigi zero","url":"bigi.html#zero"},{"doc":"bigi","title":"bigi","content":"","url":"bigi.html"}]);